# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
import time

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment, NoRangSegment

logger = logging.getLogger("driverng.hl7_SAC")


class Hl7SilSegmentMSH(HeaderSegment):
    sending_application_code = HL7String((3, 1), default="MCA")
    sending_application_libelle = HL7String((3, 2))
    receiving_application_code = HL7String((5, 1), default="")
    receiving_application_libelle = HL7String((5, 2), default="")
    message_date = HL7Date(7)
    message_type = HL7String((9, 1), default="SSU")
    message_code = HL7String((9, 2), default="U03")
    message_name = HL7String((9, 3), default="SSU_U03")
    control_id = HL7String(10)
    version = HL7String(12, default="2.5.1")
    accept_acknowledgment_type = HL7String(15, default="AL")
    application_acknowlegment_type = HL7String(16, default="NE")
    character_set = HL7String(18, default="UNICODE UTF-8")

    specimens = HL7Foreign("SPM", "header")
    queries = HL7Foreign("EQU", "header")


class Hl7SilSegmentSPM(Segment):
    # Nature id pour les patients
    # Nom du controle pour les QC
    specimen_id = HL7String(3)
    # Type de prélèvement
    specimen_type = HL7String(5)
    typ = HL7String(12, default="P")
    # Date et heure de prélèvement
    specimen_received_date = HL7Date(19)
    # Date et heure d'expiration du tube
    specimen_expiration_date = HL7String(20)

    sac_samples = HL7Foreign("SAC", "spm")
    infos = HL7Foreign("NTE", "spm")


class Hl7SilSegmentEQU(NoRangSegment):
    analyseur_id = HL7String((2, 1))
    analyseur_name = HL7String((2, 2))
    event_date = HL7Date(3)
    equipement_state = HL7String(4)
    sacs = HL7Foreign("SAC", "equ")


class Hl7SilSegmentSAC(NoRangSegment):
    # Code barre pour les patients
    # Numéro du controle pour les QC
    container_identifier = HL7String(4)
    # Rack pour les patients
    # Numéro de lot pour les QC
    rack = HL7String(11)
    position = HL7String(12)

    infos = HL7Foreign("NTE", "patient")


class Hl7SilSegmentNTE(Segment):
    # L : MCA
    # P : SIL
    source_comment = HL7String(3)
    comment = HL7String(4)


class Driver(AstmMessageV2):
    SPEC = {"code": "hl7_equsac", "label": "HL7 Tracking SIL (EQU/SAC)"}
    ENCODING = "utf-8"

    SEGMENT_CLASS_MAP = {
        "MSH": Hl7SilSegmentMSH,
        "SPM": Hl7SilSegmentSPM,
        "SAC": Hl7SilSegmentSAC,
        "NTE": Hl7SilSegmentNTE,
        "EQU": Hl7SilSegmentEQU,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    OPTIONS = [
        AstmMessageV2.Option(
            "hl7_sil_sender_code", label="Code émetteur", default="MCA"
        ),
        AstmMessageV2.Option(
            "hl7_sil_sender_label", label="Libellé émetteur", default=""
        ),
        AstmMessageV2.Option(
            "hl7_sil_receiver_code", label="Code récepteur", default=""
        ),
        AstmMessageV2.Option(
            "hl7_sil_receiver_label", label="Libellé récepteur", default=""
        ),
    ]

    def __init__(self, **kwargs):
        self.sender_code = kwargs.pop("hl7_sil_sender_code", "MCA")
        self.sender_label = kwargs.pop("hl7_sil_sender_label", "")
        self.receiver_code = kwargs.pop("hl7_sil_receiver_code", "")
        self.receiver_label = kwargs.pop("hl7_sil_receiver_label", "")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return Hl7SilSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        # Encoder un message de type résultat SIL
        if worklist.context != "info":
            raise NotImplementedError("context %s" % (worklist.context,))
        message = self.create(
            sending_application_code=self.sender_code,
            sending_application_libelle=self.sender_label,
            receiving_application_code=self.receiver_code,
            receiving_application_libelle=self.receiver_label,
            message_date=context["now"],
            control_id=context.get("control_id", int(time.time() * 1000)),
        )
        for sample in worklist.samples:
            date_plateau = sample.results[0].date_plateau if sample.results else None
            rack = position = analyseur_id = analyseur_name = ""
            if sample.rack:
                rack, position = sample.rack.rsplit("-", 1)
            if sample.request_info:
                analyseur_id, analyseur_name = sample.request_info.split("^", 1)
            else:  # tracking CKIN
                analyseur_id = sample.results[0].instrument if sample.results else None
            equ = message.queries.add(
                analyseur_id=analyseur_id,
                analyseur_name=analyseur_name,
                event_date=date_plateau or sample.request_date,
                equipement_state="OP",
            )
            equ.sacs.add(
                container_identifier=sample.sample_id, rack=rack, position=position
            )
        return message

    def decode_message(self, message):
        # TODO(eda)Décoder une info provenant du SIL
        worklist = Worklist(
            context="tracking",
            sender_id=message.sending_application_code,
            receiver_id=message.receiving_application_code,
        )
        return worklist
