# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimList as HL7List
from driverng.astm import HprimMultiple as HL7Multiple
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment

logger = logging.getLogger("driverng.hl7_ihe_pam_adt")


class HL7IHEPAMADTMSH(HeaderSegment):
    sending_application = HL7String(3)
    sending_facility = HL7String(4)
    receiving_application = HL7String(5)
    receiving_facility = HL7String(6)
    message_date = HL7Date(7)
    message_code = HL7String((9, 1))
    trigger_event = HL7String((9, 2))
    message_structure = HL7String((9, 3))
    message_control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version_id = HL7String((12, 1))
    internationalization_code = HL7String((12, 2))
    international_version_id = HL7String((12, 3))
    sequence_number = HL7String(13)
    accept_acknowledgement_type = HL7String(15)
    application_acknowledgement_type = HL7String(16)
    country_code = HL7String(17)
    character_set = HL7String(18)
    principal_language = HL7String(19)
    message_profile_identifier = HL7String(21)

    events = HL7Foreign("EVN", "header")


class HL7IHEPAMADTEVN(Segment):
    recorded_date = HL7Date(3)
    occured_date = HL7Date(7)

    patients = HL7Foreign("PID", "patient")


class HL7IHEPAMADTPID(Segment):
    identifier = HL7List(
        4,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
            assigning_facility=HL7String(6),
            effective_date=HL7Date(7),
        ),
    )
    identity = HL7List(
        6,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    mother_identity = HL7List(
        7,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    birthdate = HL7Date(8)
    sex = HL7String(9)
    address = HL7List(
        12,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
            effective_date=HL7Date(13),
            expiration_date=HL7Date(14),
        ),
    )
    telecommunication_home = HL7List(
        14,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    telecommunication_business = HL7List(
        15,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    primary_language = HL7String(16)
    marital_status = HL7String(17)
    account_number = HL7List(
        19,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )
    mother_identifier = HL7List(
        22,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
            assigning_facility=HL7String(6),
            effective_date=HL7Date(7),
        ),
    )
    multiple_birth_indicator = HL7String(25)
    birth_order = HL7String(26)
    citizenship = HL7List(
        27,
        basefield=HL7Multiple(
            0, code=HL7String(1), label=HL7String(2), coding_system=HL7String(3)
        ),
    )
    veterans_military_status = HL7String(24)
    death_date = HL7Date(30)
    death_indicator = HL7String(31)
    identity_unknow_indicator = HL7String(32)
    identity_reliability_code = HL7String(33)
    last_update_date = HL7Date(34)
    last_update_facility = HL7String(35)
    species_code = HL7String(36)
    breed_code = HL7String(37)
    strain = HL7String(38)
    production_class_code = HL7String(39)
    tribal_citizenship = HL7String(40)

    kins = HL7Foreign("NK1", "patient")
    merges = HL7Foreign("MRG", "patient")
    patients = HL7Foreign("PD1", "patient")
    roles = HL7Foreign("ROL", "patient")
    visits = HL7Foreign("PV1", "patient")


class HL7IHEPAMADTPD1(Segment):
    living_arrangement = HL7String(3)
    data_protection = HL7String(13)


class HL7IHEPAMADTROL(Segment):
    action_code = HL7String(3)
    role = HL7String(4)
    identity = HL7List(
        5,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            last_name=HL7String(2),
            first_name=HL7String(3),
            further_names=HL7String(4),
            prefix=HL7String(6),
            assigning_authority=HL7Multiple(
                9,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            name_type_code=HL7String(10),
            identifier_type_code=HL7String(13),
        ),
    )
    role_begin_date = HL7Date(6)
    role_end_date = HL7Date(7)
    role_duration = HL7String(8)
    role_action_reason = HL7String(9)
    provider_type = HL7String(10)
    organization_unit_type = HL7String(11)
    role_address = HL7List(
        12,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
            effective_date=HL7Date(13),
            expiration_date=HL7Date(14),
        ),
    )
    role_telecommunication = HL7List(
        13,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )


class HL7IHEPAMADTNK1(Segment):
    identity = HL7List(
        3,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    relationship = HL7String(4)
    address = HL7List(
        5,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
            effective_date=HL7Date(13),
            expiration_date=HL7Date(14),
        ),
    )
    telecommunication_home = HL7List(
        6,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    telecommunication_business = HL7List(
        7,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    role = HL7String(8)
    start_date = HL7Date(9)
    end_date = HL7Date(10)
    job_title = HL7String(11)
    job_code = HL7String(12)
    employee_number = HL7String(13)
    organization = HL7List(
        14,
        basefield=HL7Multiple(
            0,
            name=HL7String(1),
            assigning_authority=HL7String(6),
            type_code=HL7String(7),
            identifier=HL7String(10),
        ),
    )
    marital_status = HL7String(15)
    sex = HL7String(16)
    birthdate = HL7Date(17)
    living_dependency = HL7String(18)
    ambulatory_status = HL7String(19)
    citizenship = HL7String(20)
    language = HL7String(21)
    living_arrangement = HL7String(22)
    publicity_code = HL7String(23)
    protection_indicator = HL7String(24)
    student_indicator = HL7String(25)
    mother_identity = HL7List(
        27,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    nationality = HL7String(28)
    contact_reason = HL7String(30)
    contact_identity = HL7List(
        31,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    contact_telecommunication = HL7List(
        32,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    contact_address = HL7List(
        33,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
            effective_date=HL7Date(13),
            expiration_date=HL7Date(14),
        ),
    )
    identifier = HL7List(
        34,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )
    job_status = HL7String(35)
    handicap = HL7String(37)
    contact_ssn = HL7String(38)
    birth_place = HL7String(39)
    vip_indicator = HL7String(40)


class HL7IHEPAMADTPV1(Segment):
    patient_class = HL7String(3)
    point_of_care = HL7String((4, 1))
    room = HL7String((4, 2))
    bed = HL7String((4, 3))
    facility = HL7String((4, 4))
    bed_status = HL7String((4, 5))
    admission_type = HL7String(5)
    preadmit_number = HL7List(
        6,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )
    attending_doctors = HL7List(
        8,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            last_name=HL7String(2),
            first_name=HL7String(3),
            further_names=HL7String(4),
            prefix=HL7String(6),
            assigning_authority=HL7Multiple(
                9,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            name_type_code=HL7String(10),
            identifier_type_code=HL7String(13),
        ),
    )
    referring_doctors = HL7List(
        9,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            last_name=HL7String(2),
            first_name=HL7String(3),
            further_names=HL7String(4),
            prefix=HL7String(6),
            assigning_authority=HL7Multiple(
                9,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            name_type_code=HL7String(10),
            identifier_type_code=HL7String(13),
        ),
    )
    prior_patient_location = HL7String(10)
    hospital_service = HL7String(11)
    temporary_location = HL7String(12)
    preadmit_test_indicator = HL7String(13)
    re_admission_indicator = HL7String(14)
    admit_source = HL7String(15)
    ambulatory_status = HL7String(16)
    vip_indicator = HL7String(17)
    admitting_doctors = HL7List(
        18,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            last_name=HL7String(2),
            first_name=HL7String(3),
            further_names=HL7String(4),
            prefix=HL7String(6),
            assigning_authority=HL7Multiple(
                9,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            name_type_code=HL7String(10),
            identifier_type_code=HL7String(13),
        ),
    )
    patient_type = HL7String(19)
    visit_number = HL7List(
        20,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )
    financial_class = HL7String(21)
    charge_price_indicator = HL7String(22)
    courtesy_code = HL7String(23)
    credit_rating = HL7String(24)
    contract_code = HL7String(25)
    contract_effective_date = HL7Date(26)
    contract_amount = HL7String(27)
    contract_period = HL7String(28)
    interest_code = HL7String(29)
    transfer_to_bad_debt_code = HL7String(30)
    transfer_to_bad_debt_date = HL7Date(31)
    bad_debt_agency_code = HL7String(32)
    bad_debt_tranfer_amount = HL7String(33)
    bad_debt_recovery_amount = HL7String(34)
    delete_account_indicator = HL7String(35)
    delete_account_date = HL7Date(36)
    discharge_disposition = HL7String(37)
    discharge_to_location = HL7String(38)
    diet_type = HL7String(39)
    servicing_facility = HL7String(40)
    account_status = HL7String(42)
    admit_date = HL7Date(45)
    discharge_date = HL7Date(46)
    pending_location = HL7String(43)
    prior_temporary_location = HL7String(44)
    current_patient_balance = HL7String(47)
    total_charges = HL7String(48)
    total_adjustments = HL7String(49)
    total_payments = HL7String(50)
    alternate_visit_id = HL7List(
        51,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )
    visit_indicator = HL7String(52)

    accidents = HL7Foreign("ACC", "patient")
    demographics = HL7Foreign("ZFD", "patient")
    dmp = HL7Foreign("ZFA", "patient")
    guarantors = HL7Foreign("GT1", "patient")
    insurances = HL7Foreign("IN1", "patient")
    insurance_additionals = HL7Foreign("IN2", "patient")
    insurance_certifications = HL7Foreign("IN3", "patient")
    movements = HL7Foreign("ZBE", "patient")
    pmsi = HL7Foreign("ZFM", "patient")
    previous_facility = HL7Foreign("ZFV", "patient")
    professional_infos = HL7Foreign("ZFP", "patient")
    results = HL7Foreign("OBX", "patient")
    visits = HL7Foreign("PV2", "patient")


class HL7IHEPAMADTPV2(Segment):
    accommodation_code = HL7String(3)
    admit_reason = HL7String(4)
    transfer_reason = HL7String(5)
    patient_valuables = HL7String(6)
    patient_valuables_location = HL7String(7)
    visit_user_code = HL7String(8)
    expected_admit_date = HL7Date(9)
    expected_discharge_date = HL7Date(10)
    estimated_lenght_of_inpatient_stay = HL7String(11)
    actuel_lenght_of_inpatient_stay = HL7String(12)
    visit_description = HL7String(13)
    referral_source_code = HL7String(14)
    previous_service_date = HL7Date(15)
    employment_illness_related_indicator = HL7String(16)
    purge_status_code = HL7String(17)
    purge_status_date = HL7Date(18)
    special_program_code = HL7String(19)
    retention_indicator = HL7String(20)
    expected_number_of_insurance_plans = HL7String(21)
    visit_publicity_code = HL7String(22)
    visit_protection_indicator = HL7String(23)
    clinic_organization_name = HL7String(24)
    patient_status_code = HL7String(25)
    visit_priority_code = HL7String(26)
    previous_treatment_date = HL7Date(27)
    expected_discharge_disposition = HL7String(28)
    signature_on_file_date = HL7Date(29)
    first_similar_illness_date = HL7Date(30)
    patient_charge_ajustment_code = HL7String(31)
    recurring_service_code = HL7String(32)
    billing_media_code = HL7String(33)
    expected_surgery_date = HL7Date(34)
    military_partnership_code = HL7String(35)
    military_non_availability_code = HL7String(36)
    newborn_baby_indicator = HL7String(37)
    baby_detained_indicator = HL7String(39)
    mode_of_arrival_code = HL7String(39)
    recreational_drug_use_code = HL7String(40)
    admission_level_of_care_code = HL7String(41)
    precaution_code = HL7String(42)
    patient_condition_code = HL7String(43)
    living_will_code = HL7String(44)
    organ_donor_code = HL7String(45)
    advance_directive_code = HL7String(46)
    patient_status_effective_date = HL7Date(47)
    expected_loa_return_date = HL7Date(48)
    expected_preadmission_testing_date = HL7Date(49)
    notify_clergy_code = HL7String(50)


class HL7IHEPAMADTZBE(Segment):
    identifier = HL7List(
        2,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            namespace_id=HL7String(2),
            universal_id=HL7String(3),
            type_code=HL7String(4),
        ),
    )
    start_date = HL7Date(3)
    action = HL7String(5)
    historic_indicator = HL7String(6)
    original_trigger = HL7String(7)
    ward = HL7List(
        8,
        basefield=HL7Multiple(
            0,
            name=HL7String(1),
            assigning_authority=HL7String(6),
            type_code=HL7String(7),
            identifier=HL7String(10),
        ),
    )
    nursing_ward = HL7List(
        9,
        basefield=HL7Multiple(
            0,
            name=HL7String(1),
            assigning_authority=HL7String(6),
            type_code=HL7String(7),
            identifier=HL7String(10),
        ),
    )
    scope = HL7String(10)

    def to_string(self, separators=None):
        self.rang_segment = self.accident_date
        return super(HL7IHEPAMADTZBE, self).to_string(separators=separators)


class HL7IHEPAMADTZFA(Segment):
    dmp_status = HL7String(2)
    collecting_date = HL7Date(3)
    closing_date = HL7Date(4)
    access_authorization = HL7String(5)
    access_authorization_collecting_date = HL7Date(6)
    breakage_mode_opposition = HL7String(7)
    control_center_mode_opposition = HL7String(8)
    opposition_collecting_date = HL7Date(9)

    def to_string(self, separators=None):
        self.rang_segment = self.accident_date
        return super(HL7IHEPAMADTZFA, self).to_string(separators=separators)


class HL7IHEPAMADTZFP(Segment):
    socio_professional_activity = HL7String(2)
    socio_professional_category = HL7String(3)

    def to_string(self, separators=None):
        self.rang_segment = self.accident_date
        return super(HL7IHEPAMADTZFP, self).to_string(separators=separators)


class HL7IHEPAMADTACC(Segment):
    accident_date = HL7Date(2)
    accident_code = HL7String((3, 1))
    accident_text = HL7String((3, 2))
    accident_location = HL7String(4)
    accident_job_related_indicator = HL7String(6)
    accident_death_indicator = HL7String(7)
    entered_by = HL7List(
        8,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            last_name=HL7String(2),
            first_name=HL7String(3),
            further_names=HL7String(4),
            prefix=HL7String(6),
            assigning_authority=HL7Multiple(
                9,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            name_type_code=HL7String(10),
            identifier_type_code=HL7String(13),
        ),
    )
    accident_description = HL7String(9)
    brought_in_by = HL7String(10)
    police_notified_indicator = HL7String(11)
    accident_address = HL7List(
        12,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
        ),
    )

    def to_string(self, separators=None):
        self.rang_segment = self.accident_date
        return super(HL7IHEPAMADTACC, self).to_string(separators=separators)


class HL7IHEPAMADTZFD(Segment):
    lunar_date = HL7Date(2)
    gestation_weeks = HL7String(3)
    sms_consent = HL7String(4)
    birthdate_modified = HL7String(5)
    identity_obtaining_mode = HL7String(6)
    insi_query_date = HL7Date(7)
    identity_document = HL7String(8)
    identity_document_expiration_date = HL7Date(9)

    def to_string(self, separators=None):
        self.rang_segment = self.lunar_date
        return super(HL7IHEPAMADTZFD, self).to_string(separators=separators)


class HL7IHEPAMADTZFM(Segment):
    entry_mode = HL7String(2)
    exit_mode = HL7String(3)
    origin_mode = HL7String(4)
    destination_mode = HL7String(5)

    def to_string(self, separators=None):
        self.rang_segment = self.lunar_date
        return super(HL7IHEPAMADTZFM, self).to_string(separators=separators)


class HL7IHEPAMADTZFS(Segment):
    legal_mode_of_care_id = HL7String((3, 1))
    legal_mode_of_care_label = HL7String((3, 2))
    legal_mode_of_care_start_date = HL7Date(4)
    legal_mode_of_care_end_date = HL7Date(5)
    legal_mode_of_care__action = HL7String(6)
    legal_mode_of_care = HL7String(7)
    legal_mode_of_care_rip_p_id = HL7String(8)
    comment = HL7String(9)


class HL7IHEPAMADTZFV(Segment):
    name = HL7String((2, 1))
    entry_date = HL7Date((2, 2))
    transport_mode = HL7String(3)
    preadmission_type = HL7String(4)
    psy_start_date = HL7Date(5)
    psy_end_date = HL7Date(6)
    address = HL7List(
        7,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
        ),
    )
    nda = HL7String(8)
    archive_number = HL7String(9)
    custom_exit_mode = HL7String(10)
    rimp_code = HL7String((11, 1))
    rimp_label = HL7String((11, 2))
    transport_support_code = HL7String((12, 1))
    transport_support_label = HL7String((12, 2))

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(HL7IHEPAMADTZFV, self).to_string(separators=separators)


class HL7IHEPAMADTIN1(Segment):
    plan_id = HL7String(3)
    company_identifier = HL7List(
        4,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )
    plan_effective_date = HL7Date(13)
    plan_expiration_date = HL7Date(14)
    authorization_name = HL7String((15, 1))
    authorization_date = HL7Date((15, 2))
    exemption_from_copayment = HL7String(16)
    insured_identity = HL7List(
        17,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    beneficiary_status = HL7String(18)
    insured_address = HL7List(
        20,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
            effective_date=HL7Date(13),
            expiration_date=HL7Date(14),
        ),
    )
    third_party_payer = HL7String(21)
    insurance_nature = HL7String(32)
    company_plan_code = HL7String(36)
    event_number = HL7String(37)
    justificative_nature = HL7String(46)
    insured_identifier = HL7List(
        50,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )


class HL7IHEPAMADTIN2(Segment):
    room_coverage_type = HL7String((29, 1))
    room_coverage_amount = HL7String((29, 2))
    insured_telecommunication = HL7List(
        64,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )


class HL7IHEPAMADTIN3(Segment):
    money_or_percentage_indicator = HL7String((6, 1))
    money_or_percentage_quantity = HL7String((6, 2))


class HL7IHEPAMADTGT1(Segment):
    identifier = HL7List(
        3,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )
    identity = HL7List(
        4,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    address = HL7List(
        6,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
            effective_date=HL7Date(13),
            expiration_date=HL7Date(14),
        ),
    )
    telecommunication_home = HL7List(
        7,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    telecommunication_business = HL7List(
        8,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    type = HL7String(11)
    relationship = HL7String(12)
    ssn = HL7String(13)
    date_begin = HL7Date(14)
    date_end = HL7Date(15)
    priority = HL7String(16)
    employer_identity = HL7List(
        17,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    employer_address = HL7List(
        18,
        basefield=HL7Multiple(
            0,
            street_address=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
            effective_date=HL7Date(13),
            expiration_date=HL7Date(14),
        ),
    )
    employer_telecommunication = HL7List(
        19,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    employee_id_number = HL7String(20)
    employement_status = HL7String(21)
    organization_name = HL7String(22)
    billing_hold_flag = HL7String(23)
    credit_rating_code = HL7String(24)
    death_date = HL7Date(25)
    death_flag = HL7String(26)
    charge_ajustement_code = HL7String(27)
    household_annual_income = HL7String(28)
    household_size = HL7String(29)
    employer_identifier = HL7List(
        30,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HL7Multiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
        ),
    )
    marital_status_code = HL7String(31)
    hire_effective_date = HL7Date(32)
    employement_stop_date = HL7Date(33)
    primary_language = HL7String(37)
    living_arrangement = HL7String(38)
    nationality = HL7String(44)
    contact_identity = HL7List(
        46,
        basefield=HL7Multiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    contact_telecommunication = HL7List(
        47,
        basefield=HL7Multiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    contact_reason = HL7String(48)
    contact_relationship = HL7String(49)
    job_title = HL7String(50)
    job_code = HL7String(51)
    employer_organization = HL7List(
        52,
        basefield=HL7Multiple(
            0,
            name=HL7String(1),
            assigning_authority=HL7String(6),
            type_code=HL7String(7),
            identifier=HL7String(10),
        ),
    )
    job_status = HL7String(54)
    financial_class = HL7String(55)
    birth_place = HL7String(57)
    vip_indicator = HL7String(58)


class HL7IHEPAMADTOBX(Segment):
    value_type = HL7String(3)
    observation_identifier = HL7String((4, 1))
    observation_text = HL7String((4, 2))
    coding_system = HL7String((4, 3))
    observation_value = HL7String(6)
    units = HL7String(7)
    observation_result_status = HL7String(12)
    observation_date = HL7Date(15)
    identity = HL7List(
        17,
        basefield=HL7Multiple(
            0,
            identifier=HL7String(1),
            last_name=HL7String(2),
            first_name=HL7String(3),
            further_names=HL7String(4),
            prefix=HL7String(6),
            assigning_authority=HL7Multiple(
                9,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            name_type_code=HL7String(10),
            identifier_type_code=HL7String(13),
        ),
    )


class HL7IHEPAMADTMRG(Segment):
    id_number = HL7String((2, 1))
    check_digit = HL7String((2, 2))
    check_digit_scheme = HL7String((2, 3))
    assigning_authority = HL7String((2, 4))
    identifier_type_code = HL7String((2, 5))
    assigning_facility = HL7String((2, 6))
    effective_date = HL7Date((2, 7))
    expiration_date = HL7Date((2, 8))
    assigning_jurisdiction = HL7String((2, 9))
    assigning_department = HL7String((2, 10))

    def to_string(self, separators=None):
        self.rang_segment = self.id_number
        return super(HL7IHEPAMADTMRG, self).to_string(separators=separators)


class Driver(AstmMessageV2):
    SPEC = {"code": "hl7_ihe_pam_adt", "label": "HL7 IHE PAM ADT"}
    ENCODING = "utf-8"

    SEGMENT_CLASS_MAP = {
        "MSH": HL7IHEPAMADTMSH,
        "EVN": HL7IHEPAMADTEVN,
        "PID": HL7IHEPAMADTPID,
        "PD1": HL7IHEPAMADTPD1,
        "ROL": HL7IHEPAMADTROL,
        "NK1": HL7IHEPAMADTNK1,
        "PV1": HL7IHEPAMADTPV1,
        "PV2": HL7IHEPAMADTPV2,
        "ZBE": HL7IHEPAMADTZBE,
        "ZFA": HL7IHEPAMADTZFA,
        "ZFP": HL7IHEPAMADTZFP,
        "ACC": HL7IHEPAMADTACC,
        "ZFD": HL7IHEPAMADTZFD,
        "ZFM": HL7IHEPAMADTZFM,
        "ZFS": HL7IHEPAMADTZFS,
        "ZFV": HL7IHEPAMADTZFV,
        "IN1": HL7IHEPAMADTIN1,
        "IN2": HL7IHEPAMADTIN2,
        "IN3": HL7IHEPAMADTIN3,
        "GT1": HL7IHEPAMADTGT1,
        "OBX": HL7IHEPAMADTOBX,
        "MRG": HL7IHEPAMADTMRG,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    @staticmethod
    def get_ins_identifier(segment):
        identifiers = {
            "identifier": "",
            "status": "",
            "ipp": "",
            "oid": "",
            "authority": "",
        }
        for p in segment.identifier:
            if p.type_code in ("INS", "INS-NIR"):
                identifiers.update(
                    {
                        "identifier": p.identifier,
                        "authority": p.assigning_authority.namespace_id,
                        "oid": p.assigning_authority.universal_id,
                    }
                )
                if segment.identity_reliability_code == "VALI":
                    identifiers.update({"status": "Qualifiée"})
            elif p.type_code == "PI":
                identifiers.update({"ipp": p.identifier})
                if (
                    segment.identity_reliability_code == "PROV"
                    and identifiers.get("status") == ""
                ):
                    identifiers.update({"status": "Provisoire"})
            else:
                if identifiers.get("identifier") == "":
                    identifiers.update({"identifier": p.identifier})
                if identifiers.get("authority") == "":
                    identifiers.update(
                        {"authority": p.assigning_authority.namespace_id}
                    )
                if identifiers.get("oid") == "":
                    identifiers.update({"oid": p.assigning_authority.universal_id})
                if identifiers.get("status") == "":
                    identifiers.update({"status": "Provisoire"})
        return identifiers

    @staticmethod
    def get_other_identifier(field):
        identifiers = {"identifier": "", "oid": "", "authority": ""}
        for f in field:
            identifiers.update(
                {
                    "identifier": f.identifier,
                    "authority": f.assigning_authority.namespace_id,
                    "oid": f.assigning_authority.universal_id,
                }
            )
        return identifiers

    @staticmethod
    def get_organization(field):
        identifiers = {"name": "", "assigning_authority": "", "identifier": ""}
        for f in field:
            identifiers.update(
                {
                    "name": f.name,
                    "assigning_authority": f.assigning_authority,
                    "identifier": f.identifier,
                }
            )
        return identifiers

    @staticmethod
    def get_identity(field):
        further_names = []
        identity = {
            "last_name": "",
            "used_last_name": "",
            "first_name": "",
            "used_first_name": "",
            "further_names": [],
            "prefix": "",
            "type_code": "",
        }
        for i in field:
            if i.type_code == "L":
                identity.update(
                    {
                        "last_name": i.last_name,
                        "first_name": i.first_name,
                        "prefix": i.prefix,
                        "type_code": "normale",
                    }
                )
                if i.further_names:
                    for name in i.further_names.split(" "):
                        further_names.append(name)
                        identity.update({"further_names": further_names})
            elif i.type_code == "D":
                identity.update(
                    {"used_last_name": i.last_name, "used_first_name": i.first_name}
                )
                if identity.get("type_code") == "":
                    identity.update({"type_code": "douteuse"})
        return identity

    @staticmethod
    def get_address(field):
        address = {
            "address": "",
            "birthplace": "",
            "birthplace_code": "",
            "city": "",
            "postal_code": "",
            "county_code": "",
            "country": "",
            "effective_date": None,
            "expiration_date": None,
            "type": "",
        }
        for a in field:
            if a.type == "BDL":
                address.update({"birthplace": a.city, "birthplace_code": a.county_code})
            else:
                address.update(
                    {
                        "address": "{}~{}".format(
                            a.street_address, a.other_designation
                        ),
                        "city": a.city,
                        "county_code": a.county_code,
                        "country": a.country,
                        "postal_code": a.postal_code,
                        "effective_date": a.effective_date,
                        "expiration_date": a.expiration_date,
                        "type": a.type,
                    }
                )
        return address

    @staticmethod
    def get_telecommunications(field):
        telecommunications = {"phone": "", "email": "", "mobile": "", "phones": []}
        phones = []
        for t in field:
            if t.use_code in ("PRN", "ORN"):
                phones.append(t.phone_number)
            if t.phone_number != "" and t.equipement_type == "PH":
                telecommunications.update({"phone": t.phone_number})
            if t.phone_number != "" and t.equipement_type == "CP":
                telecommunications.update({"mobile": t.phone_number})
            if t.email != "":
                telecommunications.update({"email": t.email})
        telecommunications.update({"phones": phones})
        return telecommunications

    @staticmethod
    def get_doctor_identity(field):
        further_names = []
        identity = {
            "last_name": "",
            "first_name": "",
            "further_names": [],
            "prefix": "",
            "name_type_code": "",
            "identifier": "",
            "identifier_type_code": "",
            "oid": "",
        }
        for i in field:
            identity.update(
                {
                    "last_name": i.last_name,
                    "first_name": i.first_name,
                    "prefix": i.prefix,
                    "name_type_code": i.name_type_code,
                    "identifier": i.identifier,
                    "identifier_type_code": i.identifier_type_code,
                    "oid": i.assigning_authority.universal_id,
                }
            )
            if i.further_names:
                for name in i.further_names.split(" "):
                    further_names.append(name)
                    identity.update({"further_names": further_names})
        return identity

    @staticmethod
    def get_nk1_kins(sample, seg_pid):
        for seg_nk1 in seg_pid.kins:
            sample.relations.get_or_create(
                relationship=seg_nk1.relationship,
                role=seg_nk1.role,
                last_name=Driver.get_identity(seg_nk1.identity).get("last_name"),
                used_last_name=Driver.get_identity(seg_nk1.identity).get(
                    "used_last_name"
                ),
                first_name=Driver.get_identity(seg_nk1.identity).get("first_name"),
                used_first_name=Driver.get_identity(seg_nk1.identity).get(
                    "used_first_name"
                ),
                further_names=Driver.get_identity(seg_nk1.identity).get(
                    "further_names"
                ),
                identity_attribut=Driver.get_identity(seg_nk1.identity).get(
                    "type_code"
                ),
                title=Driver.get_identity(seg_nk1.identity).get("prefix"),
                address=Driver.get_address(seg_nk1.address).get("address"),
                city=Driver.get_address(seg_nk1.address).get("city"),
                county_code=Driver.get_address(seg_nk1.address).get("county_code"),
                country=Driver.get_address(seg_nk1.address).get("country"),
                postal_code=Driver.get_address(seg_nk1.address).get("postal_code"),
                birthplace=Driver.get_address(seg_nk1.address).get("birthplace"),
                birthplace_code=Driver.get_address(seg_nk1.address).get(
                    "birthplace_code"
                ),
                phone=Driver.get_telecommunications(seg_nk1.telecommunication_home).get(
                    "phone"
                ),
                mobile=Driver.get_telecommunications(
                    seg_nk1.telecommunication_home
                ).get("mobile"),
                email=Driver.get_telecommunications(seg_nk1.telecommunication_home).get(
                    "email"
                ),
                identifier=Driver.get_other_identifier(seg_nk1.identifier).get(
                    "identifier"
                ),
                oid=Driver.get_other_identifier(seg_nk1.identifier).get("oid"),
                pro_phone=Driver.get_telecommunications(
                    seg_nk1.telecommunication_business
                ).get("phone"),
                pro_mobile=Driver.get_telecommunications(
                    seg_nk1.telecommunication_business
                ).get("mobile"),
                pro_email=Driver.get_telecommunications(
                    seg_nk1.telecommunication_business
                ).get("email"),
                start_date=seg_nk1.start_date,
                end_date=seg_nk1.end_date,
                job_title=seg_nk1.job_title,
                job_code=seg_nk1.job_code,
                employee_number=seg_nk1.employee_number,
                organization_name=Driver.get_organization(seg_nk1.organization).get(
                    "name"
                ),
                organization_oid=Driver.get_organization(seg_nk1.organization).get(
                    "assigning_authority"
                ),
                organization_identifier=Driver.get_organization(
                    seg_nk1.organization
                ).get("identifier"),
                marital_status=seg_nk1.marital_status,
                sex=seg_nk1.sex,
                birthdate=seg_nk1.birthdate,
                living_dependency=seg_nk1.living_dependency,
                ambulatory_status=seg_nk1.ambulatory_status,
                citizenship=seg_nk1.citizenship,
                language=seg_nk1.language,
                living_arrangement=seg_nk1.living_arrangement,
                publicity_code=seg_nk1.publicity_code,
                protection_indicator=seg_nk1.protection_indicator,
                student_indicator=seg_nk1.student_indicator,
                mother_last_name=Driver.get_identity(seg_nk1.mother_identity).get(
                    "last_name"
                ),
                mother_first_name=Driver.get_identity(seg_nk1.mother_identity).get(
                    "first_name"
                ),
                mother_further_names=Driver.get_identity(seg_nk1.mother_identity).get(
                    "further_names"
                ),
                mother_prefix=Driver.get_identity(seg_nk1.mother_identity).get(
                    "prefix"
                ),
                mother_name_type_code=Driver.get_identity(seg_nk1.mother_identity).get(
                    "type_code"
                ),
                nationality=seg_nk1.nationality,
                contact_reason=seg_nk1.contact_reason,
                contact_last_name=Driver.get_identity(seg_nk1.contact_identity).get(
                    "last_name"
                ),
                contact_first_name=Driver.get_identity(seg_nk1.contact_identity).get(
                    "first_name"
                ),
                contact_further_names=Driver.get_identity(seg_nk1.contact_identity).get(
                    "further_names"
                ),
                contact_prefix=Driver.get_identity(seg_nk1.contact_identity).get(
                    "prefix"
                ),
                contact_name_type_code=Driver.get_identity(
                    seg_nk1.contact_identity
                ).get("type_code"),
                contact_phone=Driver.get_telecommunications(
                    seg_nk1.contact_telecommunication
                ).get("phone"),
                contact_mobile=Driver.get_telecommunications(
                    seg_nk1.contact_telecommunication
                ).get("mobile"),
                contact_email=Driver.get_telecommunications(
                    seg_nk1.contact_telecommunication
                ).get("email"),
                contact_address=Driver.get_address(seg_nk1.contact_address).get(
                    "address"
                ),
                contact_city=Driver.get_address(seg_nk1.contact_address).get("city"),
                contact_county_code=Driver.get_address(seg_nk1.contact_address).get(
                    "county_code"
                ),
                contact_country=Driver.get_address(seg_nk1.contact_address).get(
                    "country"
                ),
                contact_postal_code=Driver.get_address(seg_nk1.contact_address).get(
                    "postal_code"
                ),
                job_status=seg_nk1.job_status,
                handicap=seg_nk1.handicap,
                contact_ssn=seg_nk1.contact_ssn,
                birth_place=seg_nk1.birth_place,
                vip_indicator=seg_nk1.vip_indicator,
            )

    @staticmethod
    def get_pv1_doctor(sample, doctor, type=""):
        sample.physicians.get_or_create(
            type=type,
            code=doctor.identifier,
            last_name=doctor.last_name,
            first_name=doctor.first_name,
            further_names=doctor.further_names,
            title=doctor.prefix,
            codification=doctor.name_type_code,
            code_type=doctor.identifier_type_code,
            oid=doctor.assigning_authority.universal_id,
        )

    @staticmethod
    def get_zfd_demographics(sample, seg_pv1):
        for seg_zfd in seg_pv1.demographics:
            sample.update(
                lunar_date=seg_zfd.lunar_date,
                gestation_weeks=seg_zfd.gestation_weeks,
                sms_consent=seg_zfd.sms_consent,
                birthdate_modified=seg_zfd.birthdate_modified,
                identity_obtaining_mode=seg_zfd.identity_obtaining_mode,
                insi_query_date=seg_zfd.insi_query_date,
                identity_document=seg_zfd.identity_document,
                identity_document_expiration_date=seg_zfd.identity_document_expiration_date,
            )

    @staticmethod
    def get_obx_results(sample, seg_pv1):
        weight = ""
        height = ""
        for seg_obx in seg_pv1.results:
            if seg_obx.observation_identifier in ["3142-7", "8335-2", "3141-9"]:
                weight = seg_obx.observation_value
            elif seg_obx.observation_identifier in ["3137-7", "8301-4"]:
                height = seg_obx.observation_value
            sample.update(weight=weight, height=height)

    @staticmethod
    def get_pv2_visits(sample, seg_pv1):
        for seg_pv2 in seg_pv1.visits:
            sample.visit_infos.accommodation_code = seg_pv2.accommodation_code
            sample.visit_infos.visit_user_code = seg_pv2.visit_user_code
            sample.visit_infos.expected_admit_date = seg_pv2.expected_admit_date
            sample.visit_infos.expected_discharge_date = seg_pv2.expected_discharge_date
            sample.visit_infos.visit_description = seg_pv2.visit_description
            sample.visit_infos.special_program_code = seg_pv2.special_program_code
            sample.visit_infos.visit_protection_indicator = (
                seg_pv2.visit_protection_indicator
            )
            sample.visit_infos.patient_charge_ajustment_code = (
                seg_pv2.patient_charge_ajustment_code
            )
            sample.visit_infos.mode_of_arrival_code = seg_pv2.mode_of_arrival_code
            sample.visit_infos.expected_loa_return_date = (
                seg_pv2.expected_loa_return_date
            )
            sample.visit_infos.admit_reason = seg_pv2.admit_reason
            sample.visit_infos.transfer_reason = seg_pv2.transfer_reason
            sample.visit_infos.patient_valuables = seg_pv2.patient_valuables
            sample.visit_infos.patient_valuables_location = (
                seg_pv2.patient_valuables_location
            )
            sample.visit_infos.estimated_lenght_of_inpatient_stay = (
                seg_pv2.estimated_lenght_of_inpatient_stay
            )
            sample.visit_infos.actuel_lenght_of_inpatient_stay = (
                seg_pv2.actuel_lenght_of_inpatient_stay
            )
            sample.visit_infos.referral_source_code = seg_pv2.referral_source_code
            sample.visit_infos.previous_service_date = seg_pv2.previous_service_date
            sample.visit_infos.employment_illness_related_indicator = (
                seg_pv2.employment_illness_related_indicator
            )
            sample.visit_infos.purge_status_code = seg_pv2.purge_status_code
            sample.visit_infos.purge_status_code = seg_pv2.purge_status_code
            sample.visit_infos.retention_indicator = seg_pv2.retention_indicator
            sample.visit_infos.expected_number_of_insurance_plans = (
                seg_pv2.expected_number_of_insurance_plans
            )
            sample.visit_infos.visit_publicity_code = seg_pv2.visit_publicity_code
            sample.visit_infos.clinic_organization_name = (
                seg_pv2.clinic_organization_name
            )
            sample.visit_infos.patient_status_code = seg_pv2.patient_status_code
            sample.visit_infos.visit_priority_code = seg_pv2.visit_priority_code
            sample.visit_infos.previous_treatment_date = seg_pv2.previous_treatment_date
            sample.visit_infos.expected_discharge_disposition = (
                seg_pv2.expected_discharge_disposition
            )
            sample.visit_infos.signature_on_file_date = seg_pv2.signature_on_file_date
            sample.visit_infos.first_similar_illness_date = (
                seg_pv2.first_similar_illness_date
            )
            sample.visit_infos.recurring_service_code = seg_pv2.recurring_service_code
            sample.visit_infos.billing_media_code = seg_pv2.billing_media_code
            sample.visit_infos.expected_surgery_date = seg_pv2.expected_surgery_date
            sample.visit_infos.military_partnership_code = (
                seg_pv2.military_partnership_code
            )
            sample.visit_infos.military_non_availability_code = (
                seg_pv2.military_non_availability_code
            )
            sample.visit_infos.newborn_baby_indicator = seg_pv2.newborn_baby_indicator
            sample.visit_infos.baby_detained_indicator = seg_pv2.baby_detained_indicator
            sample.visit_infos.recreational_drug_use_code = (
                seg_pv2.recreational_drug_use_code
            )
            sample.visit_infos.admission_level_of_care_code = (
                seg_pv2.admission_level_of_care_code
            )
            sample.visit_infos.precaution_code = seg_pv2.precaution_code
            sample.visit_infos.patient_condition_code = seg_pv2.patient_condition_code
            sample.visit_infos.living_will_code = seg_pv2.living_will_code
            sample.visit_infos.organ_donor_code = seg_pv2.organ_donor_code
            sample.visit_infos.advance_directive_code = seg_pv2.advance_directive_code
            sample.visit_infos.patient_status_effective_date = (
                seg_pv2.patient_status_effective_date
            )
            sample.visit_infos.expected_preadmission_testing_date = (
                seg_pv2.expected_preadmission_testing_date
            )
            sample.visit_infos.notify_clergy_code = seg_pv2.notify_clergy_code

    @staticmethod
    def get_zbe_movements(sample, seg_pv1):
        for seg_zbe in seg_pv1.movements:
            for i in seg_zbe.identifier:
                identifier = i.identifier
                assigning_authority = i.namespace_id
                oid = i.universal_id
            sample.movements.identifier = identifier
            sample.movements.assigning_authority = assigning_authority
            sample.movements.oid = oid
            sample.movements.start_date = seg_zbe.start_date
            sample.movements.action = seg_zbe.action
            sample.movements.historic_indicator = seg_zbe.historic_indicator
            sample.movements.original_trigger = seg_zbe.original_trigger
            sample.movements.scope = seg_zbe.scope
            sample.movements.ward_name = Driver.get_organization(seg_zbe.ward).get(
                "name"
            )
            sample.movements.ward_assigning_authority = Driver.get_organization(
                seg_zbe.ward
            ).get("assigning_authority")
            sample.movements.ward_organization_id = Driver.get_organization(
                seg_zbe.ward
            ).get("identifier")
            sample.movements.nursing_ward_name = Driver.get_organization(
                seg_zbe.nursing_ward
            ).get("name")
            sample.movements.nursing_ward_assigning_authority = Driver.get_organization(
                seg_zbe.nursing_ward
            ).get("assigning_authority")
            sample.movements.nursing_ward_organization_id = Driver.get_organization(
                seg_zbe.nursing_ward
            ).get("identifier")

    @staticmethod
    def get_zba_dmp(sample, seg_pv1):
        for seg_zfa in seg_pv1.dmp:
            sample.dmp.dmp_status = seg_zfa.dmp_status
            sample.dmp.collecting_date = seg_zfa.collecting_date
            sample.dmp.closing_date = seg_zfa.closing_date
            sample.dmp.access_authorization = seg_zfa.access_authorization
            sample.dmp.access_authorization_collecting_date = (
                seg_zfa.access_authorization_collecting_date
            )
            sample.dmp.breakage_mode_opposition = seg_zfa.breakage_mode_opposition
            sample.dmp.control_center_mode_opposition = (
                seg_zfa.control_center_mode_opposition
            )
            sample.dmp.opposition_collecting_date = seg_zfa.opposition_collecting_date

    @staticmethod
    def get_zfp_professional_infos(sample, seg_pv1):
        for seg_zfp in seg_pv1.professional_infos:
            sample.professional_infos.socio_professional_activity = (
                seg_zfp.socio_professional_activity
            )
            sample.professional_infos.socio_professional_category = (
                seg_zfp.socio_professional_category
            )

    @staticmethod
    def get_acc_accidents(sample, seg_pv1):
        for seg_acc in seg_pv1.accidents:
            sample.accident_infos.accident_date = seg_acc.accident_date
            sample.accident_infos.accident_code = seg_acc.accident_code
            sample.accident_infos.accident_text = seg_acc.accident_text
            sample.accident_infos.accident_location = seg_acc.accident_location
            sample.accident_infos.accident_job_related_indicator = (
                seg_acc.accident_job_related_indicator
            )
            sample.accident_infos.accident_death_indicator = (
                seg_acc.accident_death_indicator
            )
            sample.accident_infos.entered_by_last_name = Driver.get_identity(
                seg_acc.entered_by
            ).get("last_name")
            sample.accident_infos.entered_by_first_name = Driver.get_identity(
                seg_acc.entered_by
            ).get("first_name")
            sample.accident_infos.entered_by_prefix = Driver.get_identity(
                seg_acc.entered_by
            ).get("prefix")
            sample.accident_infos.accident_description = seg_acc.accident_description
            sample.accident_infos.brought_in_by = seg_acc.brought_in_by
            sample.accident_infos.police_notified_indicator = (
                seg_acc.police_notified_indicator
            )
            sample.accident_infos.accident_address = Driver.get_address(
                seg_acc.accident_address
            ).get("address")
            sample.accident_infos.accident_city = Driver.get_address(
                seg_acc.accident_address
            ).get("city")
            sample.accident_infos.accident_postal_code = Driver.get_address(
                seg_acc.accident_address
            ).get("postal_code")
            sample.accident_infos.accident_country = Driver.get_address(
                seg_acc.accident_address
            ).get("country")

    @staticmethod
    def get_zfm_pmsi(sample, seg_pv1):
        for seg_zfm in seg_pv1.pmsi:
            sample.pmsi.entry_mode = seg_zfm.entry_mode
            sample.pmsi.exit_mode = seg_zfm.exit_mode
            sample.pmsi.origin_mode = seg_zfm.origin_mode
            sample.pmsi.destination_mode = seg_zfm.destination_mode

    @staticmethod
    def get_zfv_previous_facility(sample, seg_pv1):
        for seg_zfv in seg_pv1.previous_facility:
            sample.previous_facility.name = seg_zfv.name
            sample.previous_facility.entry_date = seg_zfv.entry_date
            sample.previous_facility.transport_mode = seg_zfv.transport_mode
            sample.previous_facility.psy_start_date = seg_zfv.psy_start_date
            sample.previous_facility.psy_end_date = seg_zfv.psy_end_date
            sample.previous_facility.nda = seg_zfv.nda
            sample.previous_facility.archive_number = seg_zfv.archive_number
            sample.previous_facility.custom_exit_mode = seg_zfv.custom_exit_mode
            sample.previous_facility.rimp_code = seg_zfv.rimp_code
            sample.previous_facility.rimp_label = seg_zfv.rimp_label
            sample.previous_facility.transport_support_code = (
                seg_zfv.transport_support_code
            )
            sample.previous_facility.transport_support_label = (
                seg_zfv.transport_support_label
            )
            if seg_zfv.address:
                for a in seg_zfv.address:
                    if a.type == "ORI":
                        sample.previous_facility.ori_address = "{}~{}".format(
                            a.street_address, a.other_designation
                        )
                        sample.previous_facility.ori_city = a.city
                        sample.previous_facility.ori_county_code = a.county_code
                        sample.previous_facility.ori_country = a.country
                        sample.previous_facility.ori_postal_code = a.postal_code
                    elif a.type == "DST":
                        sample.previous_facility.dst_address = "{}~{}".format(
                            a.street_address, a.other_designation
                        )
                        sample.previous_facility.dst_city = a.city
                        sample.previous_facility.dst_county_code = a.county_code
                        sample.previous_facility.dst_country = a.country
                        sample.previous_facility.dst_postal_code = a.postal_code

    @staticmethod
    def get_gt1_guarantors(sample, seg_pv1):
        for seg_gt1 in seg_pv1.guarantors:
            sample.guarantor.identifier = Driver.get_other_identifier(
                seg_gt1.identifier
            ).get("identifier")
            sample.guarantor.assigning_authority = Driver.get_other_identifier(
                seg_gt1.identifier
            ).get("authority")
            sample.guarantor.oid = Driver.get_other_identifier(seg_gt1.identifier).get(
                "oid"
            )
            sample.guarantor.last_name = Driver.get_identity(seg_gt1.identity).get(
                "last_name"
            )
            sample.guarantor.first_name = Driver.get_identity(seg_gt1.identity).get(
                "first_name"
            )
            sample.guarantor.further_names = Driver.get_identity(seg_gt1.identity).get(
                "further_names"
            )
            sample.guarantor.prefix = Driver.get_identity(seg_gt1.identity).get(
                "prefix"
            )
            sample.guarantor.name_type_code = Driver.get_identity(seg_gt1.identity).get(
                "type_code"
            )
            sample.guarantor.address = Driver.get_address(seg_gt1.address).get(
                "address"
            )
            sample.guarantor.city = Driver.get_address(seg_gt1.address).get("city")
            sample.guarantor.postal_code = Driver.get_address(seg_gt1.address).get(
                "postal_code"
            )
            sample.guarantor.country = Driver.get_address(seg_gt1.address).get(
                "country"
            )
            sample.guarantor.county_code = Driver.get_address(seg_gt1.address).get(
                "county_code"
            )
            sample.guarantor.address_type = Driver.get_address(seg_gt1.address).get(
                "type"
            )
            sample.guarantor.address_effective_date = Driver.get_address(
                seg_gt1.address
            ).get("effective_date")
            sample.guarantor.address_expiration_date = Driver.get_address(
                seg_gt1.address
            ).get("expiration_date")
            sample.guarantor.phone = Driver.get_telecommunications(
                seg_gt1.telecommunication_home
            ).get("phone")
            sample.guarantor.mobile = Driver.get_telecommunications(
                seg_gt1.telecommunication_home
            ).get("mobile")
            sample.guarantor.email = Driver.get_telecommunications(
                seg_gt1.telecommunication_home
            ).get("email")
            sample.guarantor.pro_phone = Driver.get_telecommunications(
                seg_gt1.telecommunication_business
            ).get("phone")
            sample.guarantor.pro_mobile = Driver.get_telecommunications(
                seg_gt1.telecommunication_business
            ).get("mobile")
            sample.guarantor.pro_email = Driver.get_telecommunications(
                seg_gt1.telecommunication_business
            ).get("email")
            sample.guarantor.type = seg_gt1.type
            sample.guarantor.relationship = seg_gt1.relationship
            sample.guarantor.ssn = seg_gt1.ssn
            sample.guarantor.date_begin = seg_gt1.date_begin
            sample.guarantor.date_end = seg_gt1.date_end
            sample.guarantor.priority = seg_gt1.priority
            sample.guarantor.employer_identifier = Driver.get_other_identifier(
                seg_gt1.employer_identifier
            ).get("identifier")
            sample.guarantor.employer_assigning_authority = Driver.get_other_identifier(
                seg_gt1.employer_identifier
            ).get("authority")
            sample.guarantor.employer_oid = Driver.get_other_identifier(
                seg_gt1.employer_identifier
            ).get("oid")
            sample.guarantor.employer_last_name = Driver.get_identity(
                seg_gt1.employer_identity
            ).get("last_name")
            sample.guarantor.employer_first_name = Driver.get_identity(
                seg_gt1.employer_identity
            ).get("first_name")
            sample.guarantor.employer_prefix = Driver.get_identity(
                seg_gt1.employer_identity
            ).get("prefix")
            sample.guarantor.employer_name_type_code = Driver.get_identity(
                seg_gt1.employer_identity
            ).get("type_code")
            sample.guarantor.employer_further_names = Driver.get_identity(
                seg_gt1.employer_identity
            ).get("further_names")
            sample.guarantor.employer_address = Driver.get_address(
                seg_gt1.employer_address
            ).get("address")
            sample.guarantor.employer_city = Driver.get_address(
                seg_gt1.employer_address
            ).get("city")
            sample.guarantor.employer_postal_code = Driver.get_address(
                seg_gt1.employer_address
            ).get("postal_code")
            sample.guarantor.employer_country = Driver.get_address(
                seg_gt1.employer_address
            ).get("country")
            sample.guarantor.employer_county_code = Driver.get_address(
                seg_gt1.employer_address
            ).get("county_code")
            sample.guarantor.employer_address_type = Driver.get_address(
                seg_gt1.employer_address
            ).get("type")
            sample.guarantor.employer_address_effective_date = Driver.get_address(
                seg_gt1.employer_address
            ).get("effective_date")
            sample.guarantor.employer_address_expiration_date = Driver.get_address(
                seg_gt1.employer_address
            ).get("expiration_date")
            sample.guarantor.employer_phone = Driver.get_telecommunications(
                seg_gt1.employer_telecommunication
            ).get("phone")
            sample.guarantor.employer_mobile = Driver.get_telecommunications(
                seg_gt1.employer_telecommunication
            ).get("mobile")
            sample.guarantor.employer_email = Driver.get_telecommunications(
                seg_gt1.employer_telecommunication
            ).get("email")
            sample.guarantor.employee_id_number = seg_gt1.employee_id_number
            sample.guarantor.employement_status = seg_gt1.employement_status
            sample.guarantor.organization_name = seg_gt1.organization_name
            sample.guarantor.billing_hold_flag = seg_gt1.billing_hold_flag
            sample.guarantor.credit_rating_code = seg_gt1.credit_rating_code
            sample.guarantor.death_date = seg_gt1.death_date
            sample.guarantor.death_flag = seg_gt1.death_flag
            sample.guarantor.charge_ajustement_code = seg_gt1.charge_ajustement_code
            sample.guarantor.household_annual_income = seg_gt1.household_annual_income
            sample.guarantor.household_size = seg_gt1.household_size
            sample.guarantor.marital_status_code = seg_gt1.marital_status_code
            sample.guarantor.hire_effective_date = seg_gt1.hire_effective_date
            sample.guarantor.employement_stop_date = seg_gt1.employement_stop_date
            sample.guarantor.primary_language = seg_gt1.primary_language
            sample.guarantor.living_arrangement = seg_gt1.living_arrangement
            sample.guarantor.nationality = seg_gt1.nationality
            sample.guarantor.contact_last_name = Driver.get_identity(
                seg_gt1.contact_identity
            ).get("last_name")
            sample.guarantor.contact_first_name = Driver.get_identity(
                seg_gt1.contact_identity
            ).get("first_name")
            sample.guarantor.contact_prefix = Driver.get_identity(
                seg_gt1.contact_identity
            ).get("prefix")
            sample.guarantor.contact_name_type_code = Driver.get_identity(
                seg_gt1.contact_identity
            ).get("type_code")
            sample.guarantor.contact_further_names = Driver.get_identity(
                seg_gt1.contact_identity
            ).get("further_names")
            sample.guarantor.contact_phone = Driver.get_telecommunications(
                seg_gt1.contact_telecommunication
            ).get("phone")
            sample.guarantor.contact_mobile = Driver.get_telecommunications(
                seg_gt1.contact_telecommunication
            ).get("mobile")
            sample.guarantor.contact_email = Driver.get_telecommunications(
                seg_gt1.contact_telecommunication
            ).get("email")
            sample.guarantor.contact_reason = seg_gt1.contact_reason
            sample.guarantor.contact_relationship = seg_gt1.contact_relationship
            sample.guarantor.job_title = seg_gt1.job_title
            sample.guarantor.job_code = seg_gt1.job_code
            sample.guarantor.employer_organization_name = Driver.get_organization(
                seg_gt1.employer_organization
            ).get("name")
            sample.guarantor.employer_organization_assigning_authority = Driver.get_organization(
                seg_gt1.employer_organization
            ).get(
                "assigning_authority"
            )
            sample.guarantor.employer_organization_identifier = Driver.get_organization(
                seg_gt1.employer_organization
            ).get("identifier")
            sample.guarantor.job_status = seg_gt1.job_status
            sample.guarantor.financial_class = seg_gt1.financial_class
            sample.guarantor.birth_place = seg_gt1.birth_place
            sample.guarantor.vip_indicator = seg_gt1.vip_indicator

    @staticmethod
    def get_in1_insurances(sample, seg_pv1):
        for seg_in1 in seg_pv1.insurances:
            sample.insurance.plan_id = seg_in1.plan_id
            sample.insurance.company_identifier = Driver.get_other_identifier(
                seg_in1.company_identifier
            ).get("identifier")
            sample.insurance.company_assigning_authority = Driver.get_other_identifier(
                seg_in1.company_identifier
            ).get("authority")
            sample.insurance.company_oid = Driver.get_other_identifier(
                seg_in1.company_identifier
            ).get("oid")
            sample.insurance.plan_effective_date = seg_in1.plan_effective_date
            sample.insurance.plan_expiration_date = seg_in1.plan_expiration_date
            sample.insurance.authorization_name = seg_in1.authorization_name
            sample.insurance.authorization_date = seg_in1.authorization_date
            sample.insurance.exemption_from_copayment = seg_in1.exemption_from_copayment
            sample.insurance.insured_last_name = Driver.get_identity(
                seg_in1.insured_identity
            ).get("last_name")
            sample.insurance.insured_first_name = Driver.get_identity(
                seg_in1.insured_identity
            ).get("first_name")
            sample.insurance.insured_further_names = Driver.get_identity(
                seg_in1.insured_identity
            ).get("further_names")
            sample.insurance.insured_prefix = Driver.get_identity(
                seg_in1.insured_identity
            ).get("prefix")
            sample.insurance.insured_name_type_code = Driver.get_identity(
                seg_in1.insured_identity
            ).get("type_code")
            sample.insurance.beneficiary_status = seg_in1.beneficiary_status
            sample.insurance.insured_address = Driver.get_address(
                seg_in1.insured_address
            ).get("address")
            sample.insurance.insured_city = Driver.get_address(
                seg_in1.insured_address
            ).get("city")
            sample.insurance.insured_postal_code = Driver.get_address(
                seg_in1.insured_address
            ).get("postal_code")
            sample.insurance.insured_country = Driver.get_address(
                seg_in1.insured_address
            ).get("country")
            sample.insurance.insured_address_type = Driver.get_address(
                seg_in1.insured_address
            ).get("type")
            sample.insurance.insured_county_code = Driver.get_address(
                seg_in1.insured_address
            ).get("county_code")
            sample.insurance.insured_address_effective_date = Driver.get_address(
                seg_in1.insured_address
            ).get("effective_date")
            sample.insurance.insured_address_expiration_date = Driver.get_address(
                seg_in1.insured_address
            ).get("expiration_date")
            sample.insurance.third_party_payer = seg_in1.third_party_payer
            sample.insurance.insurance_nature = seg_in1.insurance_nature
            sample.insurance.company_plan_code = seg_in1.company_plan_code
            sample.insurance.event_number = seg_in1.event_number
            sample.insurance.justificative_nature = seg_in1.justificative_nature
            sample.insurance.insured_identifier = Driver.get_other_identifier(
                seg_in1.insured_identifier
            ).get("identifier")
            sample.insurance.insured_assigning_authority = Driver.get_other_identifier(
                seg_in1.insured_identifier
            ).get("authority")
            sample.insurance.insured_oid = Driver.get_other_identifier(
                seg_in1.insured_identifier
            ).get("oid")

    @staticmethod
    def get_in2_insurance_additionals(sample, seg_pv1):
        for seg_in2 in seg_pv1.insurance_additionals:
            sample.insurance.room_coverage_type = seg_in2.room_coverage_type
            sample.insurance.room_coverage_amount = seg_in2.room_coverage_amount
            sample.insurance.insured_phone = Driver.get_telecommunications(
                seg_in2.insured_telecommunication
            ).get("phone")
            sample.insurance.insured_mobile = Driver.get_telecommunications(
                seg_in2.insured_telecommunication
            ).get("mobile")
            sample.insurance.insured_email = Driver.get_telecommunications(
                seg_in2.insured_telecommunication
            ).get("email")

    @staticmethod
    def get_in3_insurance_certifications(sample, seg_pv1):
        for seg_in3 in seg_pv1.insurance_certifications:
            sample.insurance.money_or_percentage_indicator = (
                seg_in3.money_or_percentage_indicator
            )
            sample.insurance.money_or_percentage_quantity = (
                seg_in3.money_or_percentage_quantity
            )

    def decode_message(self, message):
        events = {
            "A28": "Create patient",
            "A31": "Update patient",
            "A47": "Update patient identifiers",
            "A40": "Merge two patients",
            "A01": "Admit inpatient",
            "A04": "Register outpatient",
            "A03": "Discharge patient",
            "A05": "Pre-admit patient",
            "A06": "Change patient class to inpatient",
            "A07": "Change patient class to outpatient",
            "A02": "Transfer patient",
            "A54": "Change attending doctor",
            "A21": "Leave of absence",
            "A22": "Return from leave of absence",
            "A44": "Move account information",
            "Z99": "Admission correction",
            "A11": "Cancel admission",
            "A13": "Cancel discharge",
            "A38": "Cancel pre-admit",
            "A12": "Cancel transfer",
            "A55": "Cancel change attending doctor",
            "A52": "Cancel leave of absence",
            "A53": "Cancel return from leave of absence",
        }
        if message.trigger_event not in events:
            raise NotImplementedError

        worklist = Worklist(
            context="order",
            sender_id=message.sending_application,
            receiver_id=message.receiving_application,
        )

        for seg_evn in message.events:
            sample = worklist.get_or_create(type="patient", sample_id="")
            sample.event.type_code = message.trigger_event
            sample.event.recorded_date = seg_evn.recorded_date
            sample.event.occured_date = seg_evn.occured_date
            for key in events:
                if key == message.trigger_event:
                    label = events.get(key)
            sample.event.type_label = label

            for seg_pid in seg_evn.patients:
                citizenships = []
                for c in seg_pid.citizenship:
                    if c.code:
                        citizenships.append(c.code)

                sample.update(
                    birthdate=seg_pid.birthdate,
                    sex=seg_pid.sex,
                    language=seg_pid.primary_language,
                    marital_status=seg_pid.marital_status,
                    birth_order=seg_pid.birth_order,
                    identity_update_date=seg_pid.last_update_date,
                    patient_id=self.get_ins_identifier(seg_pid).get("ipp"),
                    ins_number=self.get_ins_identifier(seg_pid).get("identifier"),
                    oid=self.get_ins_identifier(seg_pid).get("oid"),
                    identity_status=self.get_ins_identifier(seg_pid).get("status"),
                    last_name=self.get_identity(seg_pid.identity).get("last_name"),
                    used_last_name=self.get_identity(seg_pid.identity).get(
                        "used_last_name"
                    ),
                    first_name=self.get_identity(seg_pid.identity).get("first_name"),
                    used_first_name=self.get_identity(seg_pid.identity).get(
                        "used_first_name"
                    ),
                    further_names=self.get_identity(seg_pid.identity).get(
                        "further_names"
                    ),
                    identity_attribut=self.get_identity(seg_pid.identity).get(
                        "type_code"
                    ),
                    title=self.get_identity(seg_pid.identity).get("prefix"),
                    address=self.get_address(seg_pid.address).get("address"),
                    city=self.get_address(seg_pid.address).get("city"),
                    county_code=self.get_address(seg_pid.address).get("county_code"),
                    country=self.get_address(seg_pid.address).get("country"),
                    postal_code=self.get_address(seg_pid.address).get("postal_code"),
                    address_effective_date=self.get_address(seg_pid.address).get(
                        "effective_date"
                    ),
                    address_expiration_date=self.get_address(seg_pid.address).get(
                        "expiration_date"
                    ),
                    birthplace=self.get_address(seg_pid.address).get("birthplace"),
                    birthplace_code=self.get_address(seg_pid.address).get(
                        "birthplace_code"
                    ),
                    phone=self.get_telecommunications(
                        seg_pid.telecommunication_home
                    ).get("phone"),
                    phones=self.get_telecommunications(
                        seg_pid.telecommunication_home
                    ).get("phones"),
                    mobile=self.get_telecommunications(
                        seg_pid.telecommunication_home
                    ).get("mobile"),
                    email=self.get_telecommunications(
                        seg_pid.telecommunication_home
                    ).get("email"),
                    pro_phone=self.get_telecommunications(
                        seg_pid.telecommunication_business
                    ).get("phone"),
                    pro_mobile=self.get_telecommunications(
                        seg_pid.telecommunication_business
                    ).get("mobile"),
                    pro_email=self.get_telecommunications(
                        seg_pid.telecommunication_business
                    ).get("email"),
                    citizenships=citizenships,
                    multiple_birth_indicator=seg_pid.multiple_birth_indicator,
                    veterans_military_status=seg_pid.veterans_military_status,
                    deathdate=seg_pid.death_date,
                    death_indicator=seg_pid.death_indicator,
                    identity_unknow_indicator=seg_pid.identity_unknow_indicator,
                    species_code=seg_pid.species_code,
                    breed_code=seg_pid.breed_code,
                    strain=seg_pid.strain,
                    production_class_code=seg_pid.production_class_code,
                    tribal_citizenship=seg_pid.tribal_citizenship,
                )
                if seg_pid.mother_identity:
                    sample.mother_identity.last_name = self.get_identity(
                        seg_pid.mother_identity
                    ).get("last_name")
                    sample.mother_identity.first_name = self.get_identity(
                        seg_pid.mother_identity
                    ).get("first_name")
                    sample.mother_identity.further_names = self.get_identity(
                        seg_pid.mother_identity
                    ).get("further_names")
                    sample.mother_identity.prefix = self.get_identity(
                        seg_pid.mother_identity
                    ).get("prefix")
                    sample.mother_identity.name_type_code = self.get_identity(
                        seg_pid.mother_identity
                    ).get("type_code")
                if seg_pid.account_number:
                    sample.account_number.identifier = self.get_other_identifier(
                        seg_pid.account_number
                    ).get("identifier")
                    sample.account_number.assigning_authority = self.get_other_identifier(
                        seg_pid.account_number
                    ).get(
                        "authority"
                    )
                    sample.account_number.oid = self.get_other_identifier(
                        seg_pid.account_number
                    ).get("oid")
                if seg_pid.mother_identifier:
                    sample.mother_identity.identifier = self.get_other_identifier(
                        seg_pid.mother_identifier
                    ).get("identifier")
                    sample.mother_identity.assigning_authority = self.get_other_identifier(
                        seg_pid.mother_identifier
                    ).get(
                        "authority"
                    )
                    sample.mother_identity.oid = self.get_other_identifier(
                        seg_pid.mother_identifier
                    ).get("oid")
                if seg_pid.roles:
                    physicians_codes = {
                        "AD": "admitting",
                        "AT": "attending",
                        "CP": "consulting",
                        "FHCP": "family",
                        "RP": "prescriber",
                        "RT": "referred",
                        "ODRP": "official",
                        "SUBS": "substitute",
                    }
                    for seg_rol in seg_pid.roles:
                        sample.physicians.get_or_create(
                            type=physicians_codes.get(seg_rol.role),
                            action_code=seg_rol.action_code,
                            code=self.get_doctor_identity(seg_rol.identity).get(
                                "identifier"
                            ),
                            last_name=self.get_doctor_identity(seg_rol.identity).get(
                                "last_name"
                            ),
                            first_name=self.get_doctor_identity(seg_rol.identity).get(
                                "first_name"
                            ),
                            further_names=self.get_doctor_identity(
                                seg_rol.identity
                            ).get("further_names"),
                            title=self.get_doctor_identity(seg_rol.identity).get(
                                "prefix"
                            ),
                            codification=self.get_doctor_identity(seg_rol.identity).get(
                                "name_type_code"
                            ),
                            code_type=self.get_doctor_identity(seg_rol.identity).get(
                                "identifier_type_code"
                            ),
                            oid=self.get_doctor_identity(seg_rol.identity).get("oid"),
                            begin_date=seg_rol.role_begin_date,
                            end_date=seg_rol.role_end_date,
                            role_duration=seg_rol.role_duration,
                            role_action_reason=seg_rol.role_action_reason,
                            provider_type=seg_rol.provider_type,
                            organization_unit_type=seg_rol.organization_unit_type,
                            address=self.get_address(seg_rol.role_address).get(
                                "address"
                            ),
                            city=self.get_address(seg_rol.role_address).get("city"),
                            county_code=self.get_address(seg_rol.role_address).get(
                                "county_code"
                            ),
                            country=self.get_address(seg_rol.role_address).get(
                                "country"
                            ),
                            postal_code=self.get_address(seg_rol.role_address).get(
                                "postal_code"
                            ),
                            phone=self.get_telecommunications(
                                seg_rol.role_telecommunication
                            ).get("phone"),
                            mobile=self.get_telecommunications(
                                seg_rol.role_telecommunication
                            ).get("mobile"),
                            email=self.get_telecommunications(
                                seg_rol.role_telecommunication
                            ).get("email"),
                        )
                if seg_pid.kins:
                    self.get_nk1_kins(sample, seg_pid)

                for seg_pd1 in seg_pid.patients:
                    sample.update(
                        living_arrangement=seg_pd1.living_arrangement,
                        data_protection=seg_pd1.data_protection,
                    )

                for seg_pv1 in seg_pid.visits:
                    if message.trigger_event in ("A28", "A31", "A47", "A40"):  # ITI-30
                        sample.update(patient_class=seg_pv1.patient_class)
                    else:  # ITI-31
                        sample.update(
                            patient_class=seg_pv1.patient_class,
                            location=seg_pv1.point_of_care,
                            room=seg_pv1.room,
                            bed=seg_pv1.bed,
                            hospital=seg_pv1.facility,
                            bed_status=seg_pv1.bed_status,
                            hospital_service=seg_pv1.hospital_service,
                        )
                        sample.visit_infos.admission_type = seg_pv1.admission_type
                        sample.visit_infos.preadmit_number = self.get_other_identifier(
                            seg_pv1.preadmit_number
                        ).get("identifier")
                        sample.visit_infos.preadmit_number_oid = self.get_other_identifier(
                            seg_pv1.preadmit_number
                        ).get(
                            "authority"
                        )
                        sample.visit_infos.admit_source = seg_pv1.admit_source
                        sample.visit_infos.vip_indicator = seg_pv1.vip_indicator
                        sample.visit_infos.visit_number = self.get_other_identifier(
                            seg_pv1.visit_number
                        ).get("identifier")
                        sample.visit_infos.visit_number_oid = self.get_other_identifier(
                            seg_pv1.visit_number
                        ).get("authority")
                        sample.visit_infos.financial_class = seg_pv1.financial_class
                        sample.visit_infos.charge_price_indicator = (
                            seg_pv1.charge_price_indicator
                        )
                        sample.visit_infos.courtesy_code = seg_pv1.courtesy_code
                        sample.visit_infos.discharge_disposition = (
                            seg_pv1.discharge_disposition
                        )
                        sample.visit_infos.discharge_to_location = (
                            seg_pv1.discharge_to_location
                        )
                        sample.visit_infos.account_status = seg_pv1.account_status
                        sample.visit_infos.admit_date = seg_pv1.admit_date
                        sample.visit_infos.discharge_date = seg_pv1.discharge_date
                        sample.visit_infos.prior_patient_location = (
                            seg_pv1.prior_patient_location
                        )
                        sample.visit_infos.hospital_service = seg_pv1.hospital_service
                        sample.visit_infos.temporary_location = (
                            seg_pv1.temporary_location
                        )
                        sample.visit_infos.preadmit_test_indicator = (
                            seg_pv1.preadmit_test_indicator
                        )
                        sample.visit_infos.re_admission_indicator = (
                            seg_pv1.re_admission_indicator
                        )
                        sample.visit_infos.ambulatory_status = seg_pv1.ambulatory_status
                        sample.visit_infos.patient_type = seg_pv1.patient_type
                        sample.visit_infos.credit_rating = seg_pv1.credit_rating
                        sample.visit_infos.contract_code = seg_pv1.contract_code
                        sample.visit_infos.contract_effective_date = (
                            seg_pv1.contract_effective_date
                        )
                        sample.visit_infos.contract_amount = seg_pv1.contract_amount
                        sample.visit_infos.contract_period = seg_pv1.contract_period
                        sample.visit_infos.interest_code = seg_pv1.interest_code
                        sample.visit_infos.transfer_to_bad_debt_code = (
                            seg_pv1.transfer_to_bad_debt_code
                        )
                        sample.visit_infos.transfer_to_bad_debt_date = (
                            seg_pv1.transfer_to_bad_debt_date
                        )
                        sample.visit_infos.bad_debt_agency_code = (
                            seg_pv1.bad_debt_agency_code
                        )
                        sample.visit_infos.bad_debt_tranfer_amount = (
                            seg_pv1.bad_debt_tranfer_amount
                        )
                        sample.visit_infos.bad_debt_recovery_amount = (
                            seg_pv1.bad_debt_recovery_amount
                        )
                        sample.visit_infos.delete_account_indicator = (
                            seg_pv1.delete_account_indicator
                        )
                        sample.visit_infos.delete_account_date = (
                            seg_pv1.delete_account_date
                        )
                        sample.visit_infos.diet_type = seg_pv1.diet_type
                        sample.visit_infos.servicing_facility = (
                            seg_pv1.servicing_facility
                        )
                        sample.visit_infos.pending_location = seg_pv1.pending_location
                        sample.visit_infos.prior_temporary_location = (
                            seg_pv1.prior_temporary_location
                        )
                        sample.visit_infos.admit_date = seg_pv1.admit_date
                        sample.visit_infos.discharge_date = seg_pv1.discharge_date
                        sample.visit_infos.current_patient_balance = (
                            seg_pv1.current_patient_balance
                        )
                        sample.visit_infos.total_charges = seg_pv1.total_charges
                        sample.visit_infos.total_adjustments = seg_pv1.total_adjustments
                        sample.visit_infos.total_payments = seg_pv1.total_payments
                        sample.visit_infos.alternate_visit_id = self.get_other_identifier(
                            seg_pv1.alternate_visit_id
                        ).get(
                            "identifier"
                        )
                        sample.visit_infos.alternate_visit_id_oid = self.get_other_identifier(
                            seg_pv1.alternate_visit_id
                        ).get(
                            "authority"
                        )
                        sample.visit_infos.visit_indicator = seg_pv1.visit_indicator

                    if seg_pv1.attending_doctors:
                        for attending_doctor in seg_pv1.attending_doctors:
                            self.get_pv1_doctor(
                                sample, attending_doctor, type="attending"
                            )

                    if seg_pv1.referring_doctors:
                        for referring_doctor in seg_pv1.referring_doctors:
                            self.get_pv1_doctor(
                                sample, referring_doctor, type="prescriber"
                            )

                    if seg_pv1.admitting_doctors:
                        for admitting_doctor in seg_pv1.admitting_doctors:
                            self.get_pv1_doctor(
                                sample, admitting_doctor, type="admitting"
                            )

                    if seg_pv1.demographics:
                        self.get_zfd_demographics(sample, seg_pv1)

                    if seg_pv1.results:
                        self.get_obx_results(sample, seg_pv1)

                    if seg_pv1.visits:
                        self.get_pv2_visits(sample, seg_pv1)

                    if seg_pv1.movements:
                        self.get_zbe_movements(sample, seg_pv1)

                    if seg_pv1.dmp:
                        self.get_zba_dmp(sample, seg_pv1)

                    if seg_pv1.professional_infos:
                        self.get_zfp_professional_infos(sample, seg_pv1)

                    if seg_pv1.accidents:
                        self.get_acc_accidents(sample, seg_pv1)

                    if seg_pv1.pmsi:
                        self.get_zfm_pmsi(sample, seg_pv1)

                    if seg_pv1.previous_facility:
                        self.get_zfv_previous_facility(sample, seg_pv1)

                    if seg_pv1.guarantors:
                        self.get_gt1_guarantors(sample, seg_pv1)

                    if seg_pv1.insurances:
                        self.get_in1_insurances(sample, seg_pv1)

                    if seg_pv1.insurance_additionals:
                        self.get_in2_insurance_additionals(sample, seg_pv1)

                    if seg_pv1.insurance_certifications:
                        self.get_in3_insurance_certifications(sample, seg_pv1)

                if seg_pid.merges:
                    for seg_mrg in seg_pid.merges:
                        old_ins_number = ""
                        old_ipp = ""
                        if "INS" in seg_mrg.identifier_type_code:
                            old_ins_number = seg_mrg.id_number
                        elif "PI" in seg_mrg.identifier_type_code:
                            old_ipp = seg_mrg.id_number
                        sample.update(old_ins_number=old_ins_number, old_ipp=old_ipp)

        return worklist
